searchNodes=[{"doc":"ECSx is an Entity-Component-System (ECS) framework for Elixir. In ECS: Every game object is an Entity, represented by a unique ID. The data which comprises an Entity is split among many Components. Game logic is split into Systems, which update the Components every server tick. Under the hood, ECSx uses Erlang Term Storage (ETS) to store active Components in memory. A single GenServer manages the ETS tables to ensure strict serializability and customize the run order for Systems.","ref":"ECSx.html","title":"ECSx","type":"module"},{"doc":"A Component labels an entity as having a certain attribute, and holds any data needed to model that attribute. For example, if Entities in your application should have a &quot;color&quot; value, you will create a Component type called Color . This allows you to add a color component to an Entity with add/2 , look up the color value for a given Entity with get_one/1 , get all Entities' color values with get_all/1 , remove the color value from an Entity altogether with remove/1 , or test whether an entity has a color with exists?/1 . Under the hood, we use ETS to store the Components in memory for quick retrieval via Entity ID. Usage Each Component type should have its own module, where it can be optionally configured. defmodule MyApp.Components.Color do use ECSx.Component , value : :binary , unique : true end Options :value - The type of value which will be stored in this component type. Valid types are: :atom, :binary, :datetime, :float, :integer :unique - When true , each entity may have, at most, one component of this type; attempting to add another will overwrite the first. When false , an entity may have many components of this type. :read_concurrency - when true , enables read concurrency for this component table. Only set this if you know what you're doing. Defaults to false","ref":"ECSx.Component.html","title":"ECSx.Component","type":"behaviour"},{"doc":"Creates a new component. Example # Add an ArmorRating component to entity `123` with value `10` ArmorRating . add ( 123 , 10 )","ref":"ECSx.Component.html#c:add/2","title":"ECSx.Component.add/2","type":"callback"},{"doc":"Checks if an entity has one or more components of this type.","ref":"ECSx.Component.html#c:exists?/1","title":"ECSx.Component.exists?/1","type":"callback"},{"doc":"Look up all components of this type. Example # Get all velocity components Velocity . get_all ( )","ref":"ECSx.Component.html#c:get_all/0","title":"ECSx.Component.get_all/0","type":"callback"},{"doc":"Look up all components of this type belonging to a given entity. This function is only useful for component types configured with unique: false . For unique components, get_one/1 should be used instead. Example # Get all PowerUp components for entity `123` PowerUp . get_all ( 123 )","ref":"ECSx.Component.html#c:get_all/1","title":"ECSx.Component.get_all/1","type":"callback"},{"doc":"Look up a single component. Raises if more than one component is returned. Example # Get the Velocity for entity `123` Velocity . get_one ( 123 )","ref":"ECSx.Component.html#c:get_one/1","title":"ECSx.Component.get_one/1","type":"callback"},{"doc":"Removes any existing components of this type from an entity.","ref":"ECSx.Component.html#c:remove/1","title":"ECSx.Component.remove/1","type":"callback"},{"doc":"Removes one component with a specific entity ID and value. This function is only useful for component types configured with unique: false . For unique components, remove/1 should be used instead. Example # Remove a specific PowerUp value `9` from entity `123` PowerUp . remove_one ( 123 , 9 )","ref":"ECSx.Component.html#c:remove_one/2","title":"ECSx.Component.remove_one/2","type":"callback"},{"doc":"Look up all IDs for entities which have a component of this type with a given value. Example # Get all entities with a velocity of `60` Velocity . search ( 60 )","ref":"ECSx.Component.html#c:search/1","title":"ECSx.Component.search/1","type":"callback"},{"doc":"","ref":"ECSx.Component.html#t:id/0","title":"ECSx.Component.id/0","type":"type"},{"doc":"","ref":"ECSx.Component.html#t:value/0","title":"ECSx.Component.value/0","type":"type"},{"doc":"The Manager for your ECSx application. In an ECSx application, the Manager is responsible for: starting up ETS tables for each Component Type, where the Components will be stored prepopulating the game content into memory keeping track of the Systems to run, and their run order configuring the tick rate for the application running the Systems every tick :tick_rate option The mix ecsx.setup generator creates a Manager file with :tick_rate set to 20 (i.e. 20 ticks per second). Feel free to change this number to fit the needs of your application. components/0 and systems/0 Your Manager module must contain two zero-arity functions called components and systems which return a list of all Component Types and Systems in your application. The order of the Component Types list is irrelevant, but the order of the Systems list is very important, because the Systems are run consecutively in the given order. setup block Another important piece of the Manager module is the setup block. Here you can load all the necessary data for your app before any Systems run or users connect. See setup/1 for more information.","ref":"ECSx.Manager.html","title":"ECSx.Manager","type":"module"},{"doc":"Runs the given code block during startup. The code will be run during the Manager's initialization (so pay special attention to the position of ECSx.Manager in your application's supervision tree). The Component tables will be created before setup is executed. Example defmodule YourApp.Manager do use ECSx.Manager setup do for npc &lt;- YourApp . fetch_npc_spawn_info ( ) do YourApp.Components.Name . add ( npc . id , npc . name ) YourApp.Components.HitPoints . add ( npc . id , npc . hp ) YourApp.Components.Location . add ( npc . id , npc . spawn_location ) end end end This setup will spawn each NPC with Components for Name, HitPoints, and Location.","ref":"ECSx.Manager.html#setup/1","title":"ECSx.Manager.setup/1","type":"macro"},{"doc":"A fragment of game logic which reads and updates Components. Every System must implement a run function. By default, the System will run every game tick. To use a longer period between runs, you can pass the option :period . For example, to set a System to run every 5 ticks: use ECSx.System , period : 5","ref":"ECSx.System.html","title":"ECSx.System","type":"behaviour"},{"doc":"Invoked to run System logic. This function will be called every T game ticks, where T is the value of the System's :period option (defaults to 1). Note: A crash inside this function will restart the entire app!","ref":"ECSx.System.html#c:run/0","title":"ECSx.System.run/0","type":"callback"},{"doc":"A component type which does not require a value. This is useful when the mere presence or absence of a component is all the information we need. For example, if we want a component type to model a boolean attribute, such as whether or not players may target a particular entity, we'll use a Tag: defmodule MyApp.Components.Targetable do use ECSx.Tag end Then we can check for targetability with ...Targetable.exists?(entity) or get a list of all targetable entities with ...Targetable.get_all() . Options :read_concurrency - when true , enables read concurrency for this component table. Only set this if you know what you're doing. Defaults to false","ref":"ECSx.Tag.html","title":"ECSx.Tag","type":"behaviour"},{"doc":"Creates a new tag for a given entity.","ref":"ECSx.Tag.html#c:add/1","title":"ECSx.Tag.add/1","type":"callback"},{"doc":"Checks if an entity has this tag.","ref":"ECSx.Tag.html#c:exists?/1","title":"ECSx.Tag.exists?/1","type":"callback"},{"doc":"Gets a list of all entities with this tag.","ref":"ECSx.Tag.html#c:get_all/0","title":"ECSx.Tag.get_all/0","type":"callback"},{"doc":"Removes this component from an entity.","ref":"ECSx.Tag.html#c:remove/1","title":"ECSx.Tag.remove/1","type":"callback"},{"doc":"","ref":"ECSx.Tag.html#t:id/0","title":"ECSx.Tag.id/0","type":"type"},{"doc":"Generates a new Component type for an ECSx application. $ mix ecsx.gen.component Height integer The first argument is the name of the component, followed by the data type of the value. Valid types for the component's value are: atom binary datetime float integer By default, new component types are generated with unique: true , which allows an entity to have at most one component of this type at any given time. To override this, use the --no-unique flag: $ mix ecsx.gen.component Friendship binary --no-unique","ref":"Mix.Tasks.Ecsx.Gen.Component.html","title":"Mix.Tasks.Ecsx.Gen.Component","type":"task"},{"doc":"Generates a new System for an ECSx application. $ mix ecsx.gen.system Foo The only argument accepted is a module name for the System.","ref":"Mix.Tasks.Ecsx.Gen.System.html","title":"Mix.Tasks.Ecsx.Gen.System","type":"task"},{"doc":"Generates a new Component type for an ECSx application. $ mix ecsx.gen.tag Attackable The single argument is the name of the component.","ref":"Mix.Tasks.Ecsx.Gen.Tag.html","title":"Mix.Tasks.Ecsx.Gen.Tag","type":"task"},{"doc":"Generates the Manager process which runs an ECSx application. $ mix ecsx.setup This setup will generate manager.ex and empty folders for components and systems. If you don't want to generate the folders, you can provide option --no-folders","ref":"Mix.Tasks.Ecsx.Setup.html","title":"Mix.Tasks.Ecsx.Setup","type":"task"},{"doc":"Raised at runtime when the Query is invalid.","ref":"ECSx.QueryError.html","title":"ECSx.QueryError","type":"exception"},{"doc":"To create an ECSx application, there are a few simple steps: install Elixir + erlang/OTP install Phoenix (optional) create an Elixir/Phoenix project fetch ECSx as a dependency for your project run the ECSx setup","ref":"installation.html","title":"Installation","type":"extras"},{"doc":"If you don't yet have Elixir and erlang/OTP installed on your machine, follow the instructions on the official Installation Page .","ref":"installation.html#elixir-and-erlang-otp","title":"Installation - Elixir and erlang/OTP","type":"extras"},{"doc":"If you plan on hosting your application online, you'll probably want to use Phoenix. You can skip this step if you only want to run the app locally. Otherwise, follow the instructions for Phoenix installation (the tutorial project will assume you are using Phoenix).","ref":"installation.html#phoenix","title":"Installation - Phoenix","type":"extras"},{"doc":"If you are using Phoenix, you'll create your new application with the command $ mix phx.new my_app Or for a regular Elixir application (with supervision tree): $ mix new my_app --sup","ref":"installation.html#create-project","title":"Installation - Create project","type":"extras"},{"doc":"To use the ECSx framework in your application, it should be added to the list of dependencies in my_app/mix.exs : defp deps do [ { :ecsx , &quot;~&gt; 0.3&quot; } ] end Then (from the root directory of your application) run: $ mix deps.get","ref":"installation.html#install-ecsx","title":"Installation - Install ECSx","type":"extras"},{"doc":"With ECSx installed, you can run the setup generator: $ mix ecsx.setup which will create the Manager, and two folders to get your project started. You'll also need to add the Manager to your application's supervision tree: def start ( _type , _args ) do children = [ MyApp.Manager ] opts = [ strategy : :one_for_one , name : MyApp.Supervisor ] Supervisor . start_link ( children , opts ) end","ref":"installation.html#setup-ecsx","title":"Installation - Setup ECSx","type":"extras"},{"doc":"You should now have everything you need to start building! If you're already familiar with the Entity-Component-System pattern, jump right in to the tutorial project - otherwise, start with our guide on ECS design .","ref":"installation.html#summary","title":"Installation - Summary","type":"extras"},{"doc":"","ref":"ecs_design.html","title":"ECS Design","type":"extras"},{"doc":"Everything in your application is an Entity, but in ECS you won't work with these Entities directly - instead you will work with the individual attributes that an Entity might have. These attributes are given to an Entity by creating a Component, which holds, at minimum, the Entity's unique ID, but also can store a value. For example: You're running a 2-dimensional simulation of cars on a highway Each car gets its own entity_id e.g. 123 If the car with ID 123 is blue, we give it a Color Component with value &quot;blue&quot; If the same car is moving west at 60mph, we might model this with a Direction Component with value &quot;west&quot; and a Speed Component with value 60 The car would also have Components such as XCoordinate and YCoordinate to locate it on the map","ref":"ecs_design.html#entities-and-components","title":"ECS Design - Entities and Components","type":"extras"},{"doc":"Once your Entities are modeled using Components, you'll create Systems to operate on them. For example: Entities with Speed Components should have their locations regularly updated according to the speed and direction We can create a Move System which reads the Speed and Direction Components, calculates how far the car has moved since the last server tick, and updates the Entity's XCoordinate and/or YCoordinate Component accordingly. The System will run every tick, only considering Entities which have a Speed Component","ref":"ecs_design.html#systems","title":"ECS Design - Systems","type":"extras"},{"doc":"Now that we have an Elixir project with Phoenix, we can get started on building a game with Entity-Component-System architecture. In our game, each player will control a ship, which can sail around the map, and will attack enemies if they come too close. Note: This guide will get you up-and-running with a working game, but it is intentionally generic. Feel free to experiment with altering details from this implementation to customize your own game.","ref":"tutorial.html","title":"Tutorial Project","type":"extras"},{"doc":"First let's consider the basic properties of a ship: Hull Points: How much damage can it take before it is destroyed Armor Rating: How much is each incoming attack reduced by the ship's defenses Attack Damage: How much damage does its weapon deal to enemies Attack Range: How close must enemies get before the weapon can attack Attack Speed: How much time must you wait in-between attacks X Position: The horizontal position of the ship Y Position: The vertical position of the ship X Velocity: The speed at which the ship is moving, horizontally Y Velocity: The speed at which the ship is moving, vertically Let's start by creating integer component types for each one of these, except AttackSpeed, which will use float : $ mix ecsx.gen.component HullPoints integer $ mix ecsx.gen.component ArmorRating integer $ mix ecsx.gen.component AttackDamage integer $ mix ecsx.gen.component AttackRange integer $ mix ecsx.gen.component XPosition integer $ mix ecsx.gen.component YPosition integer $ mix ecsx.gen.component XVelocity integer $ mix ecsx.gen.component YVelocity integer $ mix ecsx.gen.component AttackSpeed float For now, this is all we need to do. The ECSx generator has automatically set you up with modules for each component type, complete with a simple interface for handling the components. We'll see this in action soon.","ref":"tutorial.html#defining-component-types","title":"Tutorial Project - Defining Component Types","type":"extras"},{"doc":"Having set up the component types which will model our game data, let's think about the Systems which will organize game logic. What makes our game work? Ships change position based on velocity Ships target other ships for attack when they are within range Ships with valid targets should attack the target, reducing its hull points Ships with zero or less hull points are destroyed Players change the velocity of their ship using an input device Players can see a display of the area around their ship Let's start with changing position based on velocity. We'll call it Driver : $ mix ecsx.gen.system Driver Head over to the generated file lib/my_app/systems/driver.ex and we'll add some code: defmodule MyApp.Systems.Driver do ... use ECSx.System alias MyApp.Components.XPosition alias MyApp.Components.YPosition alias MyApp.Components.XVelocity alias MyApp.Components.YVelocity def run do for { entity , x_velocity } &lt;- XVelocity . get_all ( ) do x_position = XPosition . get_one ( entity ) new_x_position = x_position + x_velocity # By default, an entity can only have one component of each type. # Adding a second will overwrite the first. XPosition . add ( entity , new_x_position ) end # Once the x-values are updated, do the same for the y-values for { entity , y_velocity } &lt;- YVelocity . get_all ( ) do y_position = YPosition . get_one ( entity ) new_y_position = y_position + y_velocity YPosition . add ( entity , new_y_position ) end # run/0 should always return :ok :ok end end Now whenever a ship gains velocity, this system will update the position accordingly over time. Keep in mind that the velocity is relative to the server's tick rate, which by default is 20. This means the unit of measurement is &quot;game units per 1/20th of a second&quot;. For example, if you want the speed to move from XPosition 0 to XPosition 100 in one second, you divide the distance 100 by the tick rate 20, to see that an XVelocity of 5 is appropriate. The tick rate can be changed in lib/my_app/manager.ex .","ref":"tutorial.html#our-first-system","title":"Tutorial Project - Our First System","type":"extras"},{"doc":"Next let's move on to a more complicated part of the game - attacking. We'll start by considering the conditions which must be met in order to attack a given target: Target must be a ship Target must be within your ship's attack range You must not have attacked too recently (based on attack speed) For each of these conditions, we want to use the presence or absence of a component as the signal to a system that action is to be taken. For example, in the Driver system, these were the Velocity components - for each Velocity component, we made a Position update. First, for determining whether a given entity is a ship, we will simply use the existing HullPoints component, because only ships will have HullPoints. Second, for confirming the attack range, we'll make a new component type SeekingTarget which will signal to a Targeting system that a ship's proximity to other ships must be continuously calculated until a valid target is found. Then another new component type AttackTarget will replace SeekingTarget, signaling to the Targeting system that we no longer need to check for new targets. Instead, an Attacking system will detect the AttackTarget and handle the final step of the attacking process. The final attack requirement is that after a successful attack, the ship's weapon must wait for a cooldown period, based on the attack speed. To model this cooldown period, we will create an AttackCooldown component type, which will store the time at which the cooldown expires. With this plan in place, let's go ahead and create the component types, starting with SeekingTarget. Since the presence of this component alone fulfills its purpose, without the need to store additional data, this is the appropriate use-case for a Tag : $ mix ecsx.gen.tag SeekingTarget Once a target is found, the AttackTarget component will be needed, and this time a Tag will not be enough, because we need to store the ID of the target. Likewise with AttackCooldown , which must store the timestamp of the cooldown's expiration. $ mix ecsx.gen.component AttackTarget binary $ mix ecsx.gen.component AttackCooldown datetime Note: In our case, we're using binary IDs to represent Entities, and Elixir DateTime structs for cooldown expirations. If you're planning on using different types, such as integer IDs for entities, or storing timestamps as integers, simply adjust the parameters accordingly. Before we set up the systems, let's make a helper module for storing any shared mathematical logic. In particular, we'll need a function for calculating the distance between two entities. This will come in handy for several systems in the future. defmodule MyApp.SystemUtils do @moduledoc &quot;&quot;&quot; Useful math functions used by multiple systems. &quot;&quot;&quot; alias MyApp.Components.XPosition alias MyApp.Components.YPosition def distance_between ( entity_1 , entity_2 ) do x_1 = XPosition . get_one ( entity_1 ) x_2 = XPosition . get_one ( entity_2 ) y_1 = YPosition . get_one ( entity_1 ) y_2 = YPosition . get_one ( entity_2 ) x = abs ( x_1 - x_2 ) y = abs ( y_1 - y_2 ) :math . sqrt ( x * * 2 + y * * 2 ) end end Now we're onto the Targeting system, which operates only on entities with the SeekingTarget component, checking the distance to all other ships, and comparing them to the entity's attack range. When an enemy ship is found to be within range, we can remove SeekingTarget and replace it with an AttackTarget: $ mix ecsx.gen.system Targeting defmodule MyApp.Systems.Targeting do ... use ECSx.System alias MyApp.Components.AttackRange alias MyApp.Components.AttackTarget alias MyApp.Components.HullPoints alias MyApp.Components.SeekingTarget alias MyApp.SystemUtils def run do entities = SeekingTarget . get_all ( ) Enum . each ( entities , &amp; attempt_target / 1 ) end defp attempt_target ( self ) do case look_for_target ( self ) do nil -&gt; :noop { target , _hp } -&gt; add_target ( self , target ) end end defp look_for_target ( self ) do # For now, we&#39;re assuming anything which has HullPoints can be attacked HullPoints . get_all ( ) # ... except your own ship! |&gt; Enum . reject ( fn { possible_target , _hp } -&gt; possible_target == self end ) |&gt; Enum . find ( fn { possible_target , _hp } -&gt; distance_between = SystemUtils . distance_between ( possible_target , self ) range = AttackRange . get_one ( self ) distance_between &lt; range end ) end defp add_target ( self , target ) do SeekingTarget . remove ( self ) AttackTarget . add ( self , target ) end end The Attacking system will also check distance, but only to the target ship, in case it has moved out-of-range. If not, we just need to check on the cooldown, and do the attack. $ mix ecsx.gen.system Attacking defmodule MyApp.Systems.Attacking do ... use ECSx.System alias MyApp.Components.ArmorRating alias MyApp.Components.AttackCooldown alias MyApp.Components.AttackDamage alias MyApp.Components.AttackRange alias MyApp.Components.AttackSpeed alias MyApp.Components.AttackTarget alias MyApp.Components.HullPoints alias MyApp.Components.SeekingTarget alias MyApp.SystemUtils def run do attack_targets = AttackTarget . get_all ( ) Enum . each ( attack_targets , &amp; attack_if_ready / 1 ) end defp attack_if_ready ( { self , target } ) do cond do SystemUtils . distance_between ( self , target ) &gt; AttackRange . get_one ( self ) -&gt; # If the target ever leaves our attack range, we want to remove the AttackTarget # and begin searching for a new one. AttackTarget . remove ( self ) SeekingTarget . add ( self ) AttackCooldown . exists? ( self ) -&gt; # We&#39;re still within range, but waiting on the cooldown :noop :otherwise -&gt; deal_damage ( self , target ) add_cooldown ( self ) end end defp deal_damage ( self , target ) do attack_damage = AttackDamage . get_one ( self ) # Assuming one armor rating always equals one damage reduction_from_armor = ArmorRating . get_one ( target ) final_damage_amount = attack_damage - reduction_from_armor target_current_hp = HullPoints . get_one ( target ) target_new_hp = target_current_hp - final_damage_amount HullPoints . add ( target , target_new_hp ) end defp add_cooldown ( self ) do now = DateTime . utc_now ( ) ms_between_attacks = calculate_cooldown_time ( self ) cooldown_until = DateTime . add ( now , ms_between_attacks , :millisecond ) AttackCooldown . add ( self , cooldown_until ) end # We&#39;re going to model AttackSpeed with a float representing attacks per second. # The goal here is to convert that into milliseconds per attack. defp calculate_cooldown_time ( self ) do attacks_per_second = AttackSpeed . get_one ( self ) seconds_per_attack = 1 / attacks_per_second ceil ( seconds_per_attack * 1000 ) end end Phew, that was a lot! But we're still using the same basic concepts: get_all/0 to fetch the list of all relevant entities, then get_one/1 and exists?/1 to check specific attributes of the entities, and finally add/2 for creating new components, or overwriting existing ones. We're also starting to see the use of remove/1 for excluding an entity from game logic which is no longer necessary.","ref":"tutorial.html#targeting-attacking","title":"Tutorial Project - Targeting &amp; Attacking","type":"extras"},{"doc":"Our attacking system will add a cooldown with an expiration timestamp, but the next step is to ensure the cooldown component is removed from the entity once the time is reached, so it can attack again. For that, we'll create a CooldownExpiration system: $ mix ecsx.gen.system CooldownExpiration Note: going forwards, aliases will be omitted from the examples to save space. Don't forget to include the required aliases for your component types! defmodule MyApp.Systems.CooldownExpiration do ... def run do now = DateTime . utc_now ( ) cooldowns = AttackCooldown . get_all ( ) Enum . each ( cooldowns , &amp; remove_when_expired ( &amp;1 , now ) ) end defp remove_when_expired ( { entity , timestamp } , now ) do case DateTime . compare ( now , timestamp ) do :lt -&gt; :noop _ -&gt; AttackCooldown . remove ( entity ) end end end This system will check the cooldowns on each game tick, removing them as soon as the expiration time is reached.","ref":"tutorial.html#cooldowns","title":"Tutorial Project - Cooldowns","type":"extras"},{"doc":"Next let's handle what happens when a ship has its HP reduced to zero or less: $ mix ecsx.gen.component DestroyedAt datetime $ mix ecsx.gen.system Destruction defmodule MyApp.Systems.Destruction do ... def run do ships = HullPoints . get_all ( ) Enum . each ( ships , fn { entity , hp } -&gt; if hp &lt;= 0 , do : destroy ( entity ) end ) end defp destroy ( entity ) do ArmorRating . remove ( entity ) AttackCooldown . remove ( entity ) AttackDamage . remove ( entity ) AttackRange . remove ( entity ) AttackSpeed . remove ( entity ) AttackTarget . remove ( entity ) HullPoints . remove ( entity ) SeekingTarget . remove ( entity ) XPosition . remove ( entity ) XVelocity . remove ( entity ) YPosition . remove ( entity ) YVelocity . remove ( entity ) # when a ship is destroyed, other ships should stop targeting it untarget ( entity ) DestroyedAt . add ( entity , DateTime . utc_now ( ) ) end defp untarget ( target ) do for entity &lt;- AttackTarget . search ( target ) do AttackTarget . remove ( entity ) SeekingTarget . add ( entity ) end end end In this example we remove all the components the entity might have, then add a new DestroyedAt component with the current timestamp. If we wanted some components to persist - such as the position and/or velocity, so the wreckage could still be visible on the player displays - we could keep them around and possibly have another system clean them up later on. Likewise if there were other components to add, such as a RespawnTimer or FinalScore , we could add them here as well.","ref":"tutorial.html#death-destruction","title":"Tutorial Project - Death &amp; Destruction","type":"extras"},{"doc":"By now you might be wondering &quot;How did those components get created in the first place?&quot; We have code for adding AttackCooldown and DestroyedAt , when needed, but the basic components for the ships still need to be added before the game can even start. For that, we'll check out lib/my_app/manager.ex : defmodule MyApp.Manager do ... use ECSx.Manager , tick_rate : 20 setup do # Load your initial components end def components do ... end def systems do ... end end This module holds four critical pieces of data - the server's tick rate, data initialization, a list of every valid component type, and a list of each game system in the order they are to be run. Let's initialize some ship data inside the setup block: setup do for _player_count &lt;- 1 .. 4 do # First generate a unique ID to represent the new entity entity = Ecto.UUID . generate ( ) # Then use that ID to create the components which make up a ship ArmorRating . add ( entity , 0 ) AttackDamage . add ( entity , 5 ) AttackRange . add ( entity , 10 ) AttackSpeed . add ( entity , 1.05 ) HullPoints . add ( entity , 50 ) SeekingTarget . add ( entity ) XPosition . add ( entity , Enum . random ( 1 .. 100 ) ) XVelocity . add ( entity , 0 ) YPosition . add ( entity , Enum . random ( 1 .. 100 ) ) YVelocity . add ( entity , 0 ) end end Now when the server starts, there will be four ships set up and ready to go.","ref":"tutorial.html#initializing-components","title":"Tutorial Project - Initializing Components","type":"extras"},{"doc":"User input, display","ref":"tutorial.html#coming-soon","title":"Tutorial Project - Coming Soon","type":"extras"}]